% Computes the ANM series for a SYSREGTENS object
% at the given point (vector), using the given tangent vector.

% Using the "tensor" formalism:
%
%     Q is a 3-tensor, here stored as a row vector of neq1 matrices
% Q_tens = [Q1 Q2 Q3 ... Qneq1] where each "Qi" is a ninc by ninc matrix.
%
%     Qh_tens is a row vector of neqH matrices, each of size ninc by ninc.

% Note that:
%  neq1 is the number of "normal" equations in the L0/L/Q part
%  neqH is the number of equations in the Lh/Qh part
%  neq = neq1 + neqH
%  ninc = neq + 1

function [Ups,FpnlA] = calculeseries(systens,Uo,Utj)

global Q_tens Qh_tens

%% Matrice tangente systeme au point de depart
dRdU = get_dRdU(systens,Uo);

%% Vecteur tangent exact a la branche au point de depart
U1 = tangente(systens,dRdU,Uo);


if (U1'*Utj) <0
  U1 = - U1;
end

%% Matrice tangente de la MAN
K = [dRdU ; (get_chemin(systens).*U1)'];
%invK = inv(K);
[LK,UK,pK,qK] = lu(K,'vector'); % fast LU-decomposition

[neq, ninc] = size(dRdU);
neq1 = size(Q_tens,2)/ninc;
neqH = neq-neq1;

pmax = get_ordre(systens); % ANM series max. order
Ups = []; % ANM series coefficients: [Uo U1 ... Up ... Upmax]

[nbelem,nbelemh]=get_nbelem(systens); % sparse size (nnz)
% for intermediate calculation results we need to store (for speed-up)

dQdU = []; % vector of matrices where we store intermediate results concerning Q_tens
% % % dQhdU = []; % idem, concerning Qh_tens


%% 
% Point de départ (current solution point)
% 
U0=sparse(Uo);
Ups = U0; % Note that Ups(:,p+1) contains Up

%%                 
% Premier Ordre  (current tangent) 
%                 

Ups = [Ups, U1];

%%                 
% Ordres suivants 
%                 

for p=2:pmax

% Calcul du second membre à l'ordre p (computes the series coeff.)
%   
%   The calculation is done iteratively : the p_th order coeff. Up is deduced
%   from K*Up = Fpnl, where Fpnl depends only on the previous orders
%   and K is the tangent matrix of the problem that we LU-decomposed at the
%   beginning

Fpnl = zeros(ninc,1) ; % will contain the RHS

% Main calculus:
%       Fpnl = sum_{r=1}^{p-1} [ -Q(Ur,U_{p-r}) ; (p-r)/p * Qh(Ur,U_{p-r}) ]

%   Here, for Q, we sum only from r=1 to (p-1)/2 but each term contains both
%   the Q(Ur,U_{p-r}) and Q(Ur,U_{p-r}) terms  =>  "partial" sums

%    This part of the calculation is stored (sparse martices) to use it
%    when we will compute the next orders :
% The i_th column of dQdUp contains the i_th line of Q(Up,.) + Q(.,Up)
% where Up is the p_th order coeff. of the ANM series.
dQdUp=sparse([],[],[],ninc,neq1,nbelem); % partial sum : order p, Q_tens part
for i=1:neq1
    dQdUp(:,i) = (Q_tens(:,ninc*(i-1)+1:i*ninc) + Q_tens(:,ninc*(i-1)+1:i*ninc)')*Ups(:,p) ;
end
dQdU = [dQdU dQdUp]; % concatenate with the partial sums of previous orders



% ***[Sami, avril 2011]***: 
%      It seems not doable concerning Qh because of the (p-r)/p coeff. !!!
% 
% % % dQhdUp=sparse([],[],[],ninc,neqH,nbelemh); % partial sum : order p, Qh_tens part
% % % for i=1:neq-neq1
% % %    dQhdUp(:,i) = (Qh_tens(:,ninc*(i-1)+1:i*ninc) + Qh_tens(:,ninc*(i-1)+1:i*ninc)')*Ups(:,p) ;
% % % end
% % % dQhdU = [dQhdU dQhdUp]; % concatenate with previous orders



%   Now let's do the partial sums
for r=1:floor((p-1)/2)
    Fpnl(1:neq1) = Fpnl(1:neq1) - dQdU(:,(r-1)*neq1+1:r*neq1)'*Ups(:,p-r+1)  ;
end
%   When p is even, (p-1)/2 is not an integer, and the partial sum
%   does not contain the Q(U_{p/2},U_{p/2}) term, so let's 	add it here.
%   Note that we must divide by 2 because 
%       (Q(U_{p/2},.)+Q(.,U_{p/2})) * U_{p/2} = 2 Q(U_{p/2},U_{p/2]).
if mod(p,2)==0
    Fpnl(1:neq1) = Fpnl(1:neq1) -dQdU(:,(p/2-1)*neq1+1:p/2*neq1)'*Ups(:,p/2+1)/2 ;
end

% For the Qh part let's do all the job (no cum-sum...).
% This is the standard way of dealing with 3-tensors.
for r=1:p-1
    Fpnl(neq1+1:neq) = Fpnl(neq1+1:neq) + (p-r)/p*(Ups(:,p-r+1)'*reshape(Ups(:,r+1)'*Qh_tens,ninc,neqH))' ;
end


Up(qK) = UK\(LK\Fpnl(pK)) ; % fast inversion using LU
Ups = [Ups, Up'] ;
% Pour p=2 on a Ups = [U0, U1, U2]

end


%% Second membre supplementaire
FpnlA = zeros(neq,1);  

% Préparation du calcul
dQdUp=sparse([],[],[],ninc,neq,nbelem);
for i=1:neq1
    dQdUp(:,i) = (Q_tens(:,ninc*(i-1)+1:i*ninc) + Q_tens(:,ninc*(i-1)+1:i*ninc)')*Ups(:,p+1) ;
end
dQdU = [dQdU dQdUp];
% % % for i=1:neq-neq1
% % %     dQhdUp(:,i) = (Qh_tens(:,ninc*(i-1)+1:i*ninc) + Qh_tens(:,ninc*(i-1)+1:i*ninc)')*Ups(:,p+1) ;
% % % end
% % % dQhdU = [dQhdU dQhdUp];

for r=1:floor(p/2)
    FpnlA(1:neq1) = FpnlA(1:neq1)  + -dQdU(:,(r-1)*neq1+1:r*neq1)'*Ups(:,p-r+2) ;
end
if mod(p+1,2)==0
    FpnlA(1:neq1) = FpnlA(1:neq1) - dQdU(:,((p+1)/2-1)*neq1+1:(p+1)/2*neq1)'*Ups(:,(p+1)/2+1)/2 ;
end


for r=1:p
    FpnlA(neq1+1:neq) = FpnlA(neq1+1:neq) + (p+1-r)/(p+1)*(Ups(:,p+1-r+1)'*reshape(Ups(:,r+1)'*Qh_tens,ninc,neqH))' ;
end

disp(['branch sign : ',num2str(sign(prod(diag(UK))))]);
