% plotbarsincosHBM    Bar graph for Manlab/HBM 
%    Hand = plotbarsincosHBM(H,Neq,U,Idisp,fig) plots a bar graph of the
%    harmonics of the solution computed by the Manlab/HBM method.
%    It displays the amplitudes of the sine/cosine harmonic components.
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab. 
% 
%    Idisp contains the indices of the entries of the initial physical 
%    unknown vector u(t) to be displayed.
%
%    H is the number of harmonics of the simulation.
%
%    Neq is the number of equations of the initial system of unknown
%    u(t)
% 
%    fig is a figure/axis handle to plot the bargraph on.
%
%    It returns a vector of barseries handles in Hand. Each entry of
%    Hand correspond to a bar
%
%    Example: Hand=plotbarsincosHBM(obj.H,obj.Neq,U,[1 2],1)
%
%    By O. Thomas / Sept. 2010

function Hand=plotbarsincosHBM(H,Neq,U,Udisp,fig)

pencolorML;

% Column vector test
% ==================

if size(Udisp,2)>size(Udisp,1)
  Udisp=Udisp';
end

if size(U,2)>size(U,1)
  U=U';
end
if size(U,2)>1
  U=U(:,1);
  disp('tracbarU: U should be a column vector')
end

% Plotting the bargraph
% =====================

if floor(fig)==fig
  figure(fig)
  clf
  hold on
else
  axes(fig)
  hold on
end

Utrac=[];
Hand=[];
Ndisp=length(Udisp);

for iu=1:Ndisp
  udisp=Udisp(iu);
  colloc=Col(iu,:);
  Xtick=iu:Ndisp:(Ndisp*2*H+iu);
  Ubar=U(udisp:Neq:(Neq*2*H+udisp));
  hand=bar(Xtick,Ubar);
  set(hand,'Facecolor',colloc,'barwidth',.8/Ndisp)
  Hand=[Hand;hand];
  Leg{iu}=['u_',num2str(udisp)];
end

% Labels
% ======

Xtick=1:Ndisp:(2*H+1)*Ndisp;
Ntick=length(Xtick);
Xtickstr=cell(1,Ntick);
Xtickstr{1}='H0';
for ii=1:H
  Xtickstr{2*ii}=['H',num2str(ii),'c'];
  Xtickstr{2*ii+1}=['H',num2str(ii),'s'];
end

dI=2*ceil(H/6)-2;
if dI>1
  Xtickstr0=cell(1,Ntick);
  Xtickstr0(2:dI:end)=Xtickstr(2:dI:end);
  Xtickstr=Xtickstr0;
end

set(gca,'Xtick',Xtick,'XtickLabel',Xtickstr,'Xlim',[0 (2*H+1)*Ndisp+1],'Xgrid','on')
ylabel('sin/cos amplitude')
box on

legend(Hand,Leg);




return


dH=ceil(H/10);
Htick=0:dH:2*H;

dX=Ndisp*dH;
Xtick=1:dX:(2*H+1)*Ndisp;
Hsize=length(Xtick);

Xtickstr=cell(1,Hsize);
Xtickstr{1}='H0';
for ii=2:Hsize
  Xtickstr{2*ii-2}=['H',num2str(Htick(ii)),'c'];
  Xtickstr{2*ii-1}=['H',num2str(Htick(ii)),'s']; 
end

set(gca,'Xtick',Xtick,'XtickLabel',Xtickstr,'Xlim',[0 (2*H+1)*Ndisp+1],'Xgrid','on')
ylabel('Harmonics amplitude')
box on

legend(Hand,Leg);
